import React, { PropTypes } from 'react';
import { isLikeRule, style, merge } from './index.js';

export * from './index.js'; // convenience

// allows for dom elements to have a 'css' prop
export function createElement(tag, allProps) {
  for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  // todo - pull ids from className as well?
  if (typeof tag === 'string' && allProps && allProps.css) {
    var css = allProps.css;
    var props = babelHelpers.objectWithoutProperties(allProps, ['css']);

    return React.createElement.apply(React, [tag, babelHelpers.extends({}, props, Array.isArray(css) ? merge.apply(undefined, babelHelpers.toConsumableArray(css)) : isLikeRule(css) ? css : style(css))].concat(children));
  }
  return React.createElement.apply(React, [tag, allProps].concat(children));
}

export var dom = createElement;

// css vars, done right
// see examples/vars for usage
export function vars() {
  var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return function (Target) {
    var _class, _temp;

    return _temp = _class = function (_React$Component) {
      babelHelpers.inherits(Vars, _React$Component);

      function Vars() {
        babelHelpers.classCallCheck(this, Vars);
        return babelHelpers.possibleConstructorReturn(this, (Vars.__proto__ || Object.getPrototypeOf(Vars)).apply(this, arguments));
      }

      babelHelpers.createClass(Vars, [{
        key: 'getChildContext',
        value: function getChildContext() {
          return {
            glamorCssVars: babelHelpers.extends({}, this.context.glamorCssVars, typeof value === 'function' ? value(this.props) : value)
          };
        }
      }, {
        key: 'render',
        value: function render() {
          return React.createElement(Target, babelHelpers.extends({}, this.props, { vars: this.context.glamorCssVars || (typeof value === 'function' ? value(this.props) : value) }), this.props.children);
        }
      }]);
      return Vars;
    }(React.Component), _class.childContextTypes = {
      glamorCssVars: PropTypes.object
    }, _class.contextTypes = {
      glamorCssVars: PropTypes.object
    }, _temp;
  };
}

var themeIndex = 0;

export function makeTheme() {

  var key = 'data-glamor-theme-' + themeIndex++;

  var fn = function fn(_default) {
    return function (Target) {
      var _class2, _temp2;

      return _temp2 = _class2 = function (_React$Component2) {
        babelHelpers.inherits(Theme, _React$Component2);

        function Theme() {
          babelHelpers.classCallCheck(this, Theme);
          return babelHelpers.possibleConstructorReturn(this, (Theme.__proto__ || Object.getPrototypeOf(Theme)).apply(this, arguments));
        }

        babelHelpers.createClass(Theme, [{
          key: 'render',
          value: function render() {
            return React.createElement(Target, babelHelpers.extends({}, this.props, babelHelpers.defineProperty({}, key, merge.apply(undefined, [typeof _default === 'function' ? _default(this.props) : _default].concat(babelHelpers.toConsumableArray(this.context[key] || []))))));
          }
        }]);
        return Theme;
      }(React.Component), _class2.contextTypes = babelHelpers.defineProperty({}, key, PropTypes.arrayOf(PropTypes.object)), _temp2;
    };
  };

  fn.key = key;
  fn.add = function (_style) {
    return function (Target) {
      var _class3, _temp3;

      return _temp3 = _class3 = function (_React$Component3) {
        babelHelpers.inherits(ThemeOverride, _React$Component3);

        function ThemeOverride() {
          babelHelpers.classCallCheck(this, ThemeOverride);
          return babelHelpers.possibleConstructorReturn(this, (ThemeOverride.__proto__ || Object.getPrototypeOf(ThemeOverride)).apply(this, arguments));
        }

        babelHelpers.createClass(ThemeOverride, [{
          key: 'getChildContext',
          value: function getChildContext() {
            return babelHelpers.defineProperty({}, key, [typeof _style === 'function' ? _style(this.props) : _style].concat(babelHelpers.toConsumableArray(this.context[key] || [])));
          }
        }, {
          key: 'render',
          value: function render() {
            return React.createElement(Target, this.props);
          }
        }]);
        return ThemeOverride;
      }(React.Component), _class3.contextTypes = babelHelpers.defineProperty({}, key, PropTypes.arrayOf(PropTypes.object)), _class3.childContextTypes = babelHelpers.defineProperty({}, key, PropTypes.arrayOf(PropTypes.object)), _temp3;
    };
  };
  return fn;
}